-- RBT-TEST - ДПРД для функций
-- Проверка дискреционного разграничения доступа для функций
-- Права доступа для функций: EXECUTE
--
\i 'support/init.sql'
--
-- Подключение пользователя u_0_00
\connect - u_0_00
--
-- Создаем функцию и отбираем права на ее выполнение у PUBLIC
CREATE FUNCTION some_function (BOOLEAN) RETURNS TEXT AS '
    SELECT CASE WHEN $1 THEN ''success'' ELSE ''fail'' END;
' LANGUAGE sql;
REVOKE ALL ON FUNCTION some_function (BOOLEAN) FROM PUBLIC;
--
-- Создаем процедуру и отбираем права на ее выполнение у PUBLIC
CREATE PROCEDURE some_procedure () LANGUAGE plpgsql AS $$
BEGIN RAISE INFO 'Процедура'; END;
$$;
REVOKE ALL ON PROCEDURE some_procedure () FROM PUBLIC;
--
-----------------------------------------------
-- Проверка I. Владельцу все действия разрешены
-----------------------------------------------
--
--  Попытка изменить и вызвать функцию
-- Ожидается успех
ALTER FUNCTION some_function RENAME TO test_function;
SELECT test_function (true);
--
-- Попытка изменить и вызвать процедуру
-- Ожидается успех
ALTER PROCEDURE some_procedure RENAME TO notice_procedure;
CALL notice_procedure ();
--
--------------------------------------------------------------
-- Проверка II. Стороннему пользователю все действия запрещены
--------------------------------------------------------------
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка вызвать, удалить или изменить функцию
-- Ожидается ошибка доступа
SELECT test_function (true);
-- Ожидается ошибка доступа
DROP FUNCTION test_function;
-- Ожидается ошибка доступа
ALTER FUNCTION test_function RENAME TO new_test_function;
--
-- Попытка вызвать, удалить или изменить процедуру
-- Ожидается ошибка доступа
CALL notice_procedure ();
-- Ожидается ошибка доступа
DROP PROCEDURE notice_procedure;
-- Ожидается ошибка доступа
ALTER PROCEDURE notice_procedure RENAME TO new_notice_procedure;
--
--------------------------------------------------------------------------------
-- Проверка III. Передача права вызова функции/процедуры стороннему пользователю
--------------------------------------------------------------------------------
--
-- Владелец u_0_00 дает право EXECUTE пользователю u_1_10
\connect - u_0_00
GRANT EXECUTE ON FUNCTION test_function (BOOLEAN) TO u_1_10;
GRANT EXECUTE ON PROCEDURE notice_procedure () TO u_1_10;
--
-- Подключение пользователя u_1_10
\connect - u_1_10
--
-- Попытка вызвать функцию
-- Ожидается успех
SELECT test_function (true);
--
-- Попытка вызвать процедуру
-- Ожидается успех
CALL notice_procedure ();
--
-- Модификация и создание функций/процедур всё еще запрещены
-- Ожидается ошибка доступа
DROP FUNCTION test_function;
-- Ожидается ошибка доступа
ALTER FUNCTION test_function RENAME TO new_test_function;
-- Ожидается ошибка доступа
DROP PROCEDURE notice_procedure;
-- Ожидается ошибка доступа
ALTER PROCEDURE notice_procedure RENAME TO new_notice_procedure;
--
-----------------------------------------------------------------------
-- Проверка IV. Передача прав на изменение и удаление функции/процедуры
-----------------------------------------------------------------------
--
-- Включение в роль u_0_00 пользователя u2_0_00
\connect - postgres
GRANT u_0_00 TO u2_0_00;
--
-- Подключение пользователя u_1_10
\connect - u2_0_00
-- Попытка изменить, вызвать и удалить функцию
-- Ожидается успех
ALTER FUNCTION test_function RENAME TO new_test_function;
-- Ожидается успех
SELECT new_test_function (true);
-- Ожидается успех
DROP FUNCTION new_test_function;
-- Попытка изменить, вызвать и удалить процедуру
-- Ожидается успех
ALTER PROCEDURE notice_procedure RENAME TO new_notice_procedure;
-- Ожидается успех
CALL new_notice_procedure ();
-- Ожидается успех
DROP PROCEDURE new_notice_procedure;
--
\i 'support/done.sql'
