-- RBT-TEST - МПРД при декларативном секционировании
--
\set ECHO queries
-- Тестируется:
-- - Рекурсивное наследование мандатной метки родительской таблицы всеми дочерними таблицами при её изменении
-- - Рекурсивное наследование параметра CCR родительской таблицы всеми дочерними таблицами при его изменении
-- - Наследование параметра MACS родительской таблицы всеми дочерними таблицами при его изменении
-- - Запрет на смену мандатной метки дочерней таблицы
-- - Запрет на смену параметра CCR дочерней таблицы
-- - Запрет на смену параметра MACS дочерней таблицы
-- - Проверка на идентичность мандатной метки родительской таблицы и присоединяемой секционной таблицы
-- - Проверка на идентичность CCR родительской таблицы и присоединяемой секционной таблицы
-- - Проверка на идентичность MACS на строках родительской таблицы и присоединяемой секционной таблицы
-- - Проверка на запись и чтение "сверху"
-- - Проверка на запись и чтение "снизу"
-- - Удаление
-- ------------------------------------------------
--
-- Декларативное секционирование позволяет обращаться к данным как через родительскую,
-- так и через дочернюю таблицу. Поэтому необходимо, чтобы параметры МПРД родительской
-- таблицы и всех дочерних таблиц были общими.
--
-- 
--
--
\set ECHO all
-- ------------------------------------------------
--
-- Создание БД и роли, установка меток и признаков на внешние контейнеры
\set ECHO none
\i 'support/init.sql'
\i 'support/init-mac.sql'
\set ECHO all
--
-------------------------------------------
-- Этап I. Создаём таблицы для тестирования
-------------------------------------------
--
-- Родительская таблица
CREATE TABLE parent (id	INT, data TEXT) PARTITION BY RANGE (id);
-- Дочерняя таблица первого уровня (наследник родителя)
CREATE TABLE child PARTITION OF parent FOR VALUES FROM ('1') TO ('3') PARTITION BY RANGE (id);
-- Дочерняя таблица второго уровня (наследник наследника)
CREATE TABLE grandchild PARTITION OF child FOR VALUES FROM ('1') TO ('3');
-- Добавляем данные
INSERT INTO parent VALUES (1,'Text #1');
--
---------------------------------------------------------------------------------------------------------
-- Этап II. Проверяем контроль идентичности МПРД параметров родительской и присоединяемой дочерней таблиц
---------------------------------------------------------------------------------------------------------
--
-- Создаем таблицу с мандатной меткой {0,0}
CREATE TABLE adoptedchild (id INT, data TEXT);
-- Добавляем данные
INSERT INTO adoptedchild VALUES (3,'Text #3');
-- Выставляем мандатную метку присоединяемой таблицы отличной от метки родительской таблицы
MAC LABEL ON TABLE adoptedchild is '{2,2}';
--
-- Проверяем возможность добавить таблицу в группу с другой меткой {0,0}. Ожидается ошибка
ALTER TABLE parent ATTACH PARTITION adoptedchild FOR VALUES FROM ('3') TO ('5');
--
-- Выставляем мандатную метку как у группы {0,0}
MAC LABEL ON TABLE adoptedchild is '{0,0}';
--
--
-- Выставляем параметр CCR = OFF отличный от CCR = ON группы
MAC CCR ON TABLE adoptedchild is OFF;
--
-- Проверяем возможность добавить таблицу в группу с другим CCR. Ожидается ошибка
ALTER TABLE parent ATTACH PARTITION adoptedchild FOR VALUES FROM ('3') TO ('5');
--
-- Выставляем параметр CCR = ON как у группы
MAC CCR ON TABLE adoptedchild is ON;
--
--
-- Выставляем параметр WITH MACS отличный от WITHOUT MACS группы
ALTER TABLE adoptedchild SET WITH MACS;
--
-- Проверяем возможность добавить таблицу в группу с другим параметром MACS. Ожидается ошибка
ALTER TABLE parent ATTACH PARTITION adoptedchild FOR VALUES FROM ('3') TO ('5');
--
-- Выставляем параметр WITHOUT MACS как у группы
ALTER TABLE adoptedchild SET WITHOUT MACS;
--
--
-- Присоединяем таблицу с равными по отношению к родительской таблице параметрами мандатной метки, CCR, MACS
ALTER TABLE parent ATTACH PARTITION adoptedchild FOR VALUES FROM ('3') TO ('5');
-- Проверяем список дочерних таблиц в pg_inherits
SELECT inhrelid::regclass, inhparent::regclass, maclabel FROM pg_inherits WHERE 
 inhrelid IN (SELECT oid FROM pg_class WHERE relname LIKE '%child') ORDER BY inhrelid;
--
------------------------------------------------------------------------------------
-- Этап III. Проверяем наследование мандатных атрибутов родителя дочерними таблицами
------------------------------------------------------------------------------------
--
-- Выставляем новую мандатную метку родительской таблицы
MAC LABEL ON TABLE parent is '{1,0}';
-- Проверяем мандатную метку всех таблиц группы
SELECT relname, maclabel, relmacccr from pg_class WHERE relname='parent' or relname like '%child' ORDER BY relname;
-- Выставляем параметр ССR = OFF у родительской таблицы
MAC CCR ON TABLE parent is OFF;
-- Проверяем CCR всех таблиц группы
SELECT relname, relmacccr from pg_class WHERE relname='parent' or relname like '%child' ORDER BY relname;
-- Выставляем параметр ССR = ON у родительской таблицы
MAC CCR ON TABLE parent is ON;
-- Проверяем CCR всех таблиц группы
SELECT relname, relmacccr from pg_class WHERE relname='parent' or relname like '%child' ORDER BY relname;
-- Выставляем параметр WITH MACS у родительской таблицы
ALTER TABLE parent SET WITH MACS;
-- Проверяем столбец MACS у дочерних таблиц
SELECT maclabel FROM child;
SELECT maclabel FROM grandchild;
-- Выставляем параметр WITHOUT MACS у родительской таблицы
ALTER TABLE parent SET WITHOUT MACS;
-- Проверяем столбец MACS у дочерних таблиц. Ожидаются ошибки
SELECT maclabel FROM child;
SELECT maclabel FROM grandchild;
--
-------------------------------------------------------------------------
-- Этап IV. Проверяем запрет на изменение МПРД параметров дочерних таблиц
-------------------------------------------------------------------------
--
-- Попытка изменить мандатную метку дочерней таблицы первого уровня. Ожидается ошибка
MAC LABEL ON TABLE child is '{1,1}';
-- Попытка изменить мандатную метку дочерней таблицы второго уровня. Ожидается ошибка
MAC LABEL ON TABLE grandchild is '{1,1}';
-- Попытка изменить параметр CCR дочерней таблицы первого уровня. Ожидается ошибка
MAC CCR ON TABLE child is OFF;
-- Попытка изменить параметр CCR дочерней таблицы второго уровня. Ожидается ошибка
MAC CCR ON TABLE grandchild is OFF;
-- Попытка изменить параметр MACS дочерней таблицы первого уровня. Ожидается ошибка
ALTER TABLE child SET WITH MACS;
-- Попытка изменить параметр MACS дочерней таблицы второго уровня. Ожидается ошибка
ALTER TABLE grandchild SET WITH MACS;
--
------------------------------------------------------------------------------
-- Этап V. Проверяем ограничения на запись данных сверху и чтение данных снизу
------------------------------------------------------------------------------
--
-- Передаём таблицы в собственность пользователя {1,2}
ALTER TABLE parent OWNER TO u_1_10;
ALTER TABLE child OWNER TO u_1_10;
ALTER TABLE grandchild OWNER TO u_1_10;
ALTER TABLE adoptedchild OWNER TO u_1_10;
-- Проверяем возможность чтения данных
\connect mtest u_1_10
SELECT * FROM parent;
SELECT * FROM child;
SELECT * FROM grandchild;
SELECT * FROM adoptedchild;
-- Проверяем ограничение на запись данных сверху. Ожидаются ошибки
INSERT INTO parent VALUES (2,'Text #2');
INSERT INTO child VALUES (2,'Text #2');
INSERT INTO grandchild VALUES (2,'Text #2');
INSERT INTO adoptedchild VALUES (4,'Text #4');
--
--
-- Передаём таблицы в собственность пользователя {0,0}
\connect mtest postgres
ALTER TABLE parent OWNER TO u_0_00;
ALTER TABLE child OWNER TO u_0_00;
ALTER TABLE grandchild OWNER TO u_0_00;
ALTER TABLE adoptedchild OWNER TO u_0_00;
-- Меняем параметр CCR на OFF
MAC CCR ON TABLE parent is OFF;
-- Включаем метки на записях
\connect mtest postgres
ALTER TABLE parent SET WITH MACS;
-- Проверяем возможность записи пользователем {0,0}
\connect mtest u_0_00
INSERT INTO parent VALUES (4,'Text #4');
-- Проверяем возможность чтения пользователем {0,0}
\connect mtest u_0_00
SELECT *, maclabel FROM parent;
SELECT *, maclabel FROM adoptedchild;
--
-------------------------------------------------------
-- Этап VI. Проверяем удаление таблицы и завершаем тест
-------------------------------------------------------
-- Меняем параметр CCR на ON
\connect mtest postgres
MAC CCR ON TABLE parent is ON;
-- Выключаем метки на строках
ALTER TABLE parent SET WITHOUT MACS;
-- Меняем метку родительской таблицы на {1,1}
MAC LABEL ON TABLE parent is '{1,1}';
-- Передаём таблицы в собственность пользователя {1,1}
ALTER TABLE parent OWNER TO u_1_01;
ALTER TABLE child OWNER TO u_1_01;
ALTER TABLE grandchild OWNER TO u_1_01;
ALTER TABLE adoptedchild OWNER TO u_1_01;
-- Удаляем таблицу от имени пользователя {1,1}
\connect mtest u_1_01
DROP TABLE parent;
-- Проверяем что ничего не осталось
SELECT relname from pg_class WHERE relname='parent' or relname like '%child' ORDER BY relname;
--
\set ECHO errors
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'
\set ECHO all
