-- RBT-TEST - Проверка МПРД в рабочих процессах PostgreSQL
-- МПРД в рабочих процессах

\i 'support/init.sql'
\i 'support/init-mac.sql'
\i 'support/support_functions.sql'

-----------------------------------
-- 1. Проверка autovacuum launcher
-----------------------------------
-- Данный процесс должен получать доступ ко всем базам вне зависимости от их метки.
--
-- autovacuum worker запускается в следующем случае:
-- pg_stat_all_tables.n_mod_since_analyze >= autovacuum_analyze_threshold + autovacuum_analyze_scale_factor * pg_class.reltupes.
ALTER SYSTEM SET autovacuum_naptime = '1s';
ALTER SYSTEM SET autovacuum_vacuum_scale_factor = 0.03;
ALTER SYSTEM SET autovacuum_vacuum_threshold = 0;
ALTER SYSTEM SET autovacuum_analyze_scale_factor = 0.02;
ALTER SYSTEM SET autovacuum_analyze_threshold = 0;
SELECT pg_reload_conf();
-- Создание тестовой таблицы с отключенным autovacuum
CREATE TABLE autovac(
  id serial,
  s char(100)
) WITH (autovacuum_enabled = off, MACS = true);
INSERT INTO autovac SELECT g.id,'A' FROM generate_series(1,1000) g(id);
--
MAC CCR ON TABLE autovac IS OFF;
MAC LABEL ON TABLE autovac IS '{2,3}';
--
-- Таблица свежая, ожидается, что autovacuum не запускался
SELECT last_autovacuum, autovacuum_count, vacuum_count FROM pg_stat_user_tables;
--
-- 30 строк является пороговым значением из-за autovacuum_vacuum_scale_factor, равного 3%
-- Изменяем 30 строк. Ожидается, что autovacuum не сработает
UPDATE autovac SET s = 'B' WHERE id <= 30;
ALTER TABLE autovac SET (autovacuum_enabled = on);
\! sleep 5
SELECT autovacuum_count, vacuum_count FROM pg_stat_user_tables;
--
-- Изменяем 31 строку. Ожидается, что autovacuum сработает один раз
ALTER TABLE autovac SET (autovacuum_enabled = off);
UPDATE autovac SET s = 'B' WHERE id <= 31;
ALTER TABLE autovac SET (autovacuum_enabled = on);
\! sleep 5
SELECT autovacuum_count, vacuum_count FROM pg_stat_user_tables;
--
-- Изменяем 31 строку. Ожидается, что autovacuum сработает второй раз
ALTER TABLE autovac SET (autovacuum_enabled = off);
CHMAC autovac SET maclabel = '{1,1}' WHERE id <= 31;
ALTER TABLE autovac SET (autovacuum_enabled = on);
\! sleep 5
SELECT autovacuum_count, vacuum_count FROM pg_stat_user_tables;
-- Очистка после теста
DROP TABLE autovac;
ALTER SYSTEM SET autovacuum_naptime = DEFAULT;
ALTER SYSTEM SET autovacuum_vacuum_scale_factor = DEFAULT;
ALTER SYSTEM SET autovacuum_vacuum_threshold = DEFAULT;
ALTER SYSTEM SET autovacuum_analyze_scale_factor = DEFAULT;
ALTER SYSTEM SET autovacuum_analyze_threshold = DEFAULT;
SELECT pg_reload_conf();
--
-----------------------------------
-- 2. Проверка autovacuum worker
-----------------------------------
-- autovacuum worker должен получить доступ ко всем таблицам базы без ограничений
-- Проверка этого утверждения производится в пункте 1.
--
-----------------------------------
-- 3. Проверка logger
-----------------------------------
-- Данный процесс не осуществляет попыток доступа к данным. Не взаимодействует с пользователем. Все взаимодействие происхоит с postmaster через IPC.
--
-----------------------------------
-- 4. Проверка checkpointer
-----------------------------------
-- Записывает "грязные" кеши на диск. Имеет доступ ко всем данным в кешах, игнорируя разграничение доступа. Не взаимодействует с пользователем.
--
ALTER SYSTEM SET checkpoint_completion_target = 0.0001;
ALTER SYSTEM SET log_checkpoints = on;
ALTER SYSTEM SET checkpoint_timeout = '30s';
SELECT pg_reload_conf();

CREATE EXTENSION pg_buffercache;
SELECT pg_stat_reset_shared('bgwriter');
CHECKPOINT;
-- Ожидается 0, т.к. после CHECKPOINT
SELECT count(*) FROM pg_buffercache buf, pg_class c WHERE buf.relfilenode = c.relfilenode AND isdirty AND c.relnamespace != (SELECT oid FROM pg_namespace WHERE nspname = 'pg_catalog');
-- checkpoints_timed ожидается 0
SELECT checkpoints_timed FROM pg_stat_bgwriter \gx
-- Генерируем данные
CREATE TABLE chkpt AS SELECT * FROM generate_series(1,100000) AS g(n);
-- Ожидается ненулевое значение, т.к. данные в буферном кеше
SELECT (SELECT count(*) FROM pg_buffercache buf, pg_class c WHERE buf.relfilenode = c.relfilenode AND c.relname = 'chkpt' and isdirty) > 0;
-- checkpoints_timed ожидается 0
SELECT checkpoints_timed FROM pg_stat_bgwriter \gx
-- Ожидается 0 после срабатывания автоматического checkpointer
SELECT * FROM advanced_waiting(query := 'SELECT count(*) FROM pg_buffercache buf, pg_class c WHERE buf.relfilenode = c.relfilenode AND c.relname = ''chkpt'' AND isdirty', 
                               expected_result := 0,
                               label := 'Ожидается 0 после срабатывания автоматического checkpointer');
-- checkpoints_timed ожидается 1
SELECT checkpoints_timed FROM pg_stat_bgwriter \gx

-- Очистка после теста
DROP EXTENSION pg_buffercache;
DROP TABLE chkpt;
ALTER SYSTEM SET checkpoint_completion_target = DEFAULT;
ALTER SYSTEM SET log_checkpoints = DEFAULT;
ALTER SYSTEM SET checkpoint_timeout = DEFAULT;
SELECT pg_reload_conf();
--
-----------------------------------
-- 5. Проверка background writer
-----------------------------------
-- Записывает "грязные" кеши на диск, которые вероятнее всего подлежат вытеснению из кеша (имеют нулевое число обращений). Не взаимодействует с пользователем.
--
-----------------------------------
-- 6. Проверка walwriter
-----------------------------------
--
create extension pageinspect;
create table wal(id integer) with (autovacuum_enabled = off);
mac label on table wal is '{2,2}';
-- проверка записи wal вручную
do $$                                        
declare
  old_wal text := null;
  new_wal text := null;
  old_lsn pg_lsn := null;
  new_lsn pg_lsn := null;
  new_ins_lsn pg_lsn:= null;
begin
  checkpoint;
  old_lsn := pg_current_wal_lsn();
  old_wal := pg_walfile_name(old_lsn);
  insert into wal values (1);
  new_lsn := pg_current_wal_lsn();
  new_ins_lsn := pg_current_wal_insert_lsn();
  new_wal := pg_walfile_name(new_lsn);
  case
    when (new_wal = old_wal) and (new_lsn > old_lsn) then
      raise notice 'Проверка записи WAL после вставки данных - успешно.';
    when (new_wal = old_wal) and (new_ins_lsn > new_lsn) then
      raise notice 'Проверка записи WAL после вставки данных - успешно.';
    else
      raise notice 'Проверка записи WAL после вставки данных - провал. old_wal - %, old_lsn - %, new_wal - %, new_lsn - %, new_ins_lsn - %', old_wal, old_lsn, new_wal, new_lsn, new_ins_lsn;
  end case;
  --
  old_lsn := pg_current_wal_lsn();
  old_wal := pg_walfile_name(old_lsn);
  checkpoint;
  new_lsn := pg_current_wal_lsn();
  new_ins_lsn := pg_current_wal_insert_lsn();
  new_wal := pg_walfile_name(new_lsn);
  if (new_wal = old_wal) and ((new_lsn = old_lsn) or (new_ins_lsn = new_lsn)) then
    raise notice 'Проверка записи WAL после контрольной точки - успешно.';
  else
    raise notice 'Проверка записи WAL после контрольной точки - провал. old_wal - %, old_lsn - %, new_wal - %, new_lsn - %, new_ins_lsn - %', old_wal, old_lsn, new_wal, new_lsn, new_ins_lsn;
  end if;
  --
  alter table wal set with macs;
  mac ccr on table wal is off;
  checkpoint;
  old_lsn := pg_current_wal_lsn();
  old_wal := pg_walfile_name(old_lsn);
  update wal set id = id + 1;
  new_lsn := pg_current_wal_lsn();
  new_ins_lsn := pg_current_wal_insert_lsn();
  new_wal := pg_walfile_name(new_lsn);
  case
    when (new_wal = old_wal) and (new_lsn > old_lsn) then
      raise notice 'Проверка записи WAL после изменения данных - успешно.';
    when (new_wal = old_wal) and (new_ins_lsn > new_lsn) then
      raise notice 'Проверка записи WAL после изменения данных - успешно.';
    else
      raise notice 'Проверка записи WAL после изменения данных - провал. old_wal - %, old_lsn - %, new_wal - %, new_lsn - %, new_ins_lsn - %', old_wal, old_lsn, new_wal, new_lsn, new_ins_lsn;
  end case;
  --
  checkpoint;
  old_lsn := pg_current_wal_lsn();
  old_wal := pg_walfile_name(old_lsn);
  delete from wal;
  new_lsn := pg_current_wal_lsn();
  new_ins_lsn := pg_current_wal_insert_lsn();
  new_wal := pg_walfile_name(new_lsn);
  case
    when (new_wal = old_wal) and (new_lsn > old_lsn) then
      raise notice 'Проверка записи WAL после удаления данных - успешно.';
    when (new_wal = old_wal) and (new_ins_lsn > new_lsn) then
      raise notice 'Проверка записи WAL после удаления данных - успешно.';
    else
      raise notice 'Проверка записи WAL после удаления данных - провал. old_wal - %, old_lsn - %, new_wal - %, new_lsn - %, new_ins_lsn - %', old_wal, old_lsn, new_wal, new_lsn, new_ins_lsn;
  end case;
end $$;

-- Проверка записи WAL процессом walwriter
ALTER SYSTEM SET wal_writer_delay = '20ms';
SELECT pg_reload_conf();
BEGIN;
    INSERT INTO wal VALUES (23);
    CHECKPOINT;
    -- Ожидается нулевое значение, т.к. CHECKPOINT сбросил кеши WAL
    SELECT * FROM advanced_waiting(query := 'SELECT (pg_current_wal_insert_lsn() - pg_current_wal_lsn())', 
                                   expected_result := 0,
                                   label := 'Проверка записи WAL процессом walwriter. Ожидается нулевое значение, т.к. CHECKPOINT сбросил кеши WAL');
    --
    CHMAC wal SET maclabel = '{1,2}' WHERE id = 23;
    -- Ожидается ненулевое значение.
    SELECT * FROM advanced_waiting(query := 'SELECT ((pg_current_wal_insert_lsn() - pg_current_wal_lsn()) > 0)::int', 
                                   expected_result := 1,
                                   label := 'Проверка записи WAL процессом walwriter. Ожидается ненулевое значение.');
    -- Значение 0 означает, что walwriter записал кеши на диск по таймеру
    SELECT * FROM advanced_waiting(query := 'SELECT (pg_current_wal_insert_lsn() - pg_current_wal_lsn())', 
                                   expected_result := 0,
                                   label := 'Проверка записи WAL процессом walwriter. Значение 0 означает, что walwriter записал кеши на диск по таймеру');
END;
-- Возвращаем исходное значение wal_writer_delay из postgresql.conf
\! grep wal_writer_delay /etc/postgresql/$PG_VERSION/$PG_SETEST_CLUSTER/postgresql.conf
ALTER SYSTEM SET wal_writer_delay = '200ms';
SELECT pg_reload_conf();

DROP EXTENSION pageinspect;
--
-----------------------------------
-- 7. Проверка stats collector
-----------------------------------
-- Работа данного процесса проверена в пункте 1, т.к. перед автоматическим запуском vacuum должен произойти анализ количества измененных строк в таблице.
--
-----------------------------------
-- 8. Проверка logical replication launcher
-----------------------------------
-- Проверяется в mac-replication-logical
-----------------------------------
-- 9. Проверка logical replication worker
-----------------------------------
-- Проверяется в mac-replication-logical
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'