-- RBT-TEST - HOT обновления
-- Проверка выключения HOT оптимизации при работе с метками безопасности
-- Отключение HOT для операций CHMAC необходимо для своевременной актуализации
-- меток записей в индексе таблицы с метками на строках
--
---------------------------
-- Этап I. Подготовительный
---------------------------
\i 'support/init.sql'
\i 'support/init-mac.sql'
--
-- Подключаемся как пользователь postgres
\connect - postgres
--
-- Устанавливаем уровень сессии
SET AC_SESSION_MACLABEL='{2,3}';
--
-- Создаём расширение pageinspect
CREATE EXTENSION pageinspect;
--
-- Создаем проверочную таблицу с защитой строк
CREATE TABLE tt1 (id int, data int) WITH (MACS=TRUE);
--
-- Создаем индекс с защитой строк
CREATE INDEX tt1_ix ON tt1 (id) WITH (MACS=TRUE);
--
-- Создаём функцию для триггера
CREATE OR REPLACE FUNCTION fun_update()
  RETURNS trigger AS
$BODY$
begin
	if(new.id = 1) THEN -- строка обновляется без изменения метки
		new.data=new.data + 1;
	end if;
	if(new.id = 2) THEN -- строка обновляется с изменением метки (без команды chmac)
		new.data=new.data + 1;
		new.maclabel='{1,0}';
	end if;
return new;
END
$BODY$
LANGUAGE plpgsql;
--
-- Создаём триггер на таблицу
CREATE TRIGGER tg_BR
  BEFORE UPDATE
  ON tt1
  FOR EACH ROW
  EXECUTE PROCEDURE fun_update();
--
-- Создаём функцию обработки данных, полученных со страницы таблицы
CREATE FUNCTION heap_page(relname text, pageno integer)
RETURNS TABLE(ctid tid, t_ctid tid, state text, HEAP_HOT_UPDATED text, HEAP_ONLY_TUPLE text, data bytea, maclabel text)
AS $$
SELECT (pageno,lp)::text::tid AS ctid,
       t_ctid,
       CASE lp_flags
         WHEN 0 THEN 'unused'
         WHEN 1 THEN 'normal'
         WHEN 2 THEN 'redirect to '||lp_off
         WHEN 3 THEN 'dead'
       END AS state,
       CASE WHEN (t_infomask2 & 16384) > 0 THEN 'true' END AS HEAP_HOT_UPDATED,
       CASE WHEN (t_infomask2 & 32768) > 0 THEN 'true' END AS HEAP_ONLY_TUPLE,
       t_data as data,
       t_maclabel
FROM heap_page_items(get_raw_page(relname,pageno))
ORDER BY lp;
$$ LANGUAGE SQL;
--
-- Создаём представление для сопоставления записей таблицы и индекса
CREATE VIEW compare_rel_index AS
	SELECT *,
		CASE WHEN "Метка в индексе" = "Метка в таблице" THEN 'true' END AS "Равнозначность мандаметок"
	 FROM
		(SELECT ctid AS "{Cтраница, Положение}",
			t_ctid AS "Указатель {Cтраница, Положение}",
			HEAP_HOT_UPDATED AS "Обновление HoT",
			HEAP_ONLY_TUPLE AS "Без индекса",
			get_byte(data, 4) AS "data",
			state AS "Состояние",
			maclabel AS "Метка в таблице"
		FROM heap_page('tt1',0)) t
		FULL OUTER JOIN
		(SELECT ctid AS "Указатель в индексе",
				maclabel AS "Метка в индексе"
		FROM bt_page_items('tt1_ix', 1)) i
	ON t."{Cтраница, Положение}"=i."Указатель в индексе";
--
-------------------------------------------
-- Этап II. Проверка работы HOT оптимизации
-------------------------------------------
--
-- Наполняем проверочную таблицу данными
INSERT INTO tt1 (id, data) VALUES (1, 1), (2, 10), (3, 100);
--
-- Данные в таблице до обновлений
SELECT data, maclabel FROM tt1 ORDER BY id;
--
-- Выполним обновление первой и второй строки таблицы, дополнительно используя функцию триггера
UPDATE tt1 SET data = data + 1 WHERE id = 1 OR id = 2;
--
-- Выполним обновление метки безопасности третьей строки
CHMAC tt1 SET maclabel='{2,0}' WHERE id = 3;
--
-- Данные в таблице после обновлений
SELECT data, maclabel FROM tt1 ORDER BY id;
--
-- Ожидается, что механизм оптимизации обновления "HEAP_ONLY_TUPLE", исключающий создание новой записи
-- в индексе, будет задействован только при обновлении первой строки, в котором метка не изменилась
SELECT 	"data",
	"{Cтраница, Положение}",
	"Указатель {Cтраница, Положение}",
	"Обновление HoT",
	"Без индекса",
	"Равнозначность мандаметок"
 FROM compare_rel_index;
--
-- Выполним очистку
VACUUM tt1;
--
-- Перепроверим статус записей после очистки
SELECT "data",
	"{Cтраница, Положение}",
	"Указатель {Cтраница, Положение}",
	"Обновление HoT",
	"Без индекса",
	"Равнозначность мандаметок",
	"Состояние"
 FROM compare_rel_index;
--
-----------------------
-- Этап III. Завершение
-----------------------
--
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'
