-- RBT-TEST - Проверка БД в режиме киоска
-- Проверка БД в режиме киоска (запрет на создание и изменений функций и процедур)
--
\i 'support/init.sql'
\i 'support/init-config-tools.sql'
--
-- Подключаемся суперпользователем postgres
\connect mtest postgres
--
-- Создаем функции для последующий попытки их изменения (ожидается успех)
CREATE FUNCTION functest(int) RETURNS bool LANGUAGE 'sql' AS 'SELECT $1 > 0';
CREATE PROCEDURE proctest(int) LANGUAGE SQL AS $$ $$;
CREATE AGGREGATE agg_new(sfunc1=int4pl, basetype=int4, stype1=int4, initcond=0);
-- Создаём объект, использующий range функцию, для последующей попытки изменения (ожидается успех)
CREATE TYPE textrange_new AS range(subtype=text, collation="C");
-- Включаем режим киоска
SELECT ac_change_param('ac_kiosk_mode', true);
\! sleep 1
-- Пытаемся создать новые функции (ожидается провал)
CREATE FUNCTION functest_2(int) RETURNS bool LANGUAGE 'sql' AS 'SELECT $1 > 0';
CREATE PROCEDURE proctest_2(int) LANGUAGE SQL AS $$ $$;
CREATE AGGREGATE aggtest_2(sfunc1=int4pl, basetype=int4, stype1=int4, initcond=0);
-- Пытаемся изменить ранее созданные функции (ожидается провал)
ALTER FUNCTION functest RETURNS NULL ON NULL INPUT;
ALTER PROCEDURE proctest RESET search_path;
ALTER AGGREGATE agg_new(int4) RENAME TO aggtest;
-- Пытаемся напрямую изменить функцию "functest" при помощи модификации записи в таблице "pg_proc" (ожидается провал)
UPDATE pg_proc SET prosrc = 'NEW_FUNCTION_BODY' WHERE proname = 'functest';
DELETE FROM pg_proc WHERE proname = 'functest';
-- Убеждаемся, что функция "functest" не была изменена, и что записи "pg_proc" доступны для чтения (ожидается успех)
SELECT proname, prosrc FROM pg_proc WHERE proname = 'functest';
-- Пытаемся удалить созданные и несозданные функции (ожидается провал)
DROP FUNCTION functest;
DROP FUNCTION functest_2;
DROP PROCEDURE proctest;
DROP PROCEDURE proctest_2;
DROP AGGREGATE agg_new(int4);
DROP AGGREGATE aggtest(int4);
DROP AGGREGATE aggtest_2(int4);
-- Пытаемся создать новый объект, использующий range функцию, и изменить старый (ожидается успех)
CREATE TYPE textrange_c_2 AS range(subtype=text, collation="C");
ALTER TYPE textrange_new RENAME TO textrange_c;
DROP TYPE textrange_c;
DROP TYPE textrange_c_2;
-- Пытаемся создать функцию напрямую, путём вставки записи в таблицу "pg_proc" (ожидается провал)
-- SELECT ac_change_param('ac_kiosk_mode', true);
-- \! sleep 2
-- INSERT INTO pg_proc(
--     proname, prosrc, oid, pronamespace, proowner, prolang, procost, prorows, provariadic, prosupport, prokind,
--     prosecdef, proleakproof, proisstrict, proretset, provolatile, proparallel, pronargs, pronargdefaults,
--     prorettype, proargtypes  -- TODO: см. ниже
-- )
-- VALUES (
--     'functest_2', '_', 9999, 1, 2, 3, 4, 5, 6, 0, 'f',
--     false, false, false, false, 'i', 'u', 0, 0,
--     1, [1, 2]::oidvector  -- TODO: непонятно как задать oidvector, может быть это в целом невозможно сделать из SQL
-- );
--
-- Выключаем режим киоска
SELECT ac_change_param('ac_kiosk_mode', false);
--
\i 'support/done.sql'
