-- RBT-TEST - Проверка аутентификации pam
-- Проверка ограничений на создание пароля
-- Пароль должен состоять из слова длиной не менее 8 символов
-- Для пароля должен быть доступен выбор не менее 70 уникальных символов
--
------------------------------------------------------------------------------------
-- Этап I. Подготовительный. Создаём БД, пользователей и меняем метод аутентификации
------------------------------------------------------------------------------------
\i 'support/init.sql'
\i 'support/init-mac.sql'
--
--
-- Включаем аутентификацию "pam" для роли "u_0_01"
\! sed  -i '4 i\local	all		u_0_01		pam' /etc/postgresql/$PG_VERSION/$PG_SETEST_CLUSTER/pg_hba.conf
--
-- Применяем настройки
SELECT pg_reload_conf();
--
-- Задаём первоначальный пароль для пользователя "u_0_01" в ОС администратором
\! echo -e  'frstpswd13\nfrstpswd13' | passwd u_0_01
--
-- Проверяем необходимость указывания пароля. Ожидается ошибка из-за неверного пароля
\! PGPASSWORD="aa" psql -p $PG_SETEST_PORT -U u_0_01 -d mtest -c "show ac_session_maclabel;"
--
-- Подключаемся с правильным паролем. Ожидается успех
\! PGPASSWORD="frstpswd13" psql -p $PG_SETEST_PORT -U u_0_01 -d mtest -c "show ac_session_maclabel;"
--
----------------------------------------------
-- Этап II. Проверяем систему изменения пароля
----------------------------------------------
--
-- Пользователь пробует поменять свой пароль на новый вариант Sd9{Z32 из 7 символов
-- Ожидается ошибка: слишком простой пароль
\! su u_0_01 -c "echo -e 'frstpswd13\nSd9{Z32\nSd9{Z32'| passwd"
--
-- Пользователь меняет пароль на вариант из 85 уникальных символов
-- Ожидается успех
\! su u_0_01 -c "echo -e 'frstpswd13\nx8)a(2R@EGNlwtorV4C]K.LUZ+iuQBvpj?5kc6D}:|\;X[bs&#mhAy9P0zTg^fWd$7J{MFnY,e<OS%3>qH_=-1I\nx8)a(2R@EGNlwtorV4C]K.LUZ+iuQBvpj?5kc6D}:|\;X[bs&#mhAy9P0zTg^fWd$7J{MFnY,e<OS%3>qH_=-1I'| passwd"
--
-- Подключаемся с новым надёжным паролем. Ожидается успех
\! PGPASSWORD="x8)a(2R@EGNlwtorV4C]K.LUZ+iuQBvpj?5kc6D}:|\;X[bs&#mhAy9P0zTg^fWd$7J{MFnY,e<OS%3>qH_=-1I" psql -p $PG_SETEST_PORT -U u_0_01 -d mtest -c "show ac_session_maclabel;"
--
--------------------
-- Этап III. Очистка
--------------------
--
-- Удаляем настройку pam из pg_hba
\! sed -i '4d' /etc/postgresql/$PG_VERSION/$PG_SETEST_CLUSTER/pg_hba.conf
--
\i 'support/done.sql'
\i 'support/reset-cluster-mac-attributes.sql'
