#!/bin/bash

DATABASE_NAME=mtest
PGDATA=/sefiles
PORT=6002
#$PG_SETEST_PORT

# get major version of Postgres
PG_VERSION=$(psql -p $PORT -d $DATABASE_NAME -U postgres -t -c "select floor((select current_setting('server_version_num')::DECIMAL)/10000);" |  sed 's/ //g')

# get CATALOG_VERSION_NO (catversion.h)
CATALOG_VERSION_NO=$(psql -p $PORT -d $DATABASE_NAME -U postgres -t -c "SELECT catalog_version_no FROM pg_control_system();" |  sed 's/ //g')

get_file_mac()
{
    if test -f $1; then
        pdpl-file $1
    else
        echo "file does not exist"
    fi
}

if [ $# -eq 2 ]
then
    PGDATA=$2
fi

# get database information
database_info=$( /usr/lib/postgresql/${PG_VERSION}/bin/oid2name -U postgres -H localhost -p $PORT -q | grep $DATABASE_NAME )
database_oid=$( echo $database_info | awk '{print $1}' )
database_tablespace=$( echo $database_info | awk '{print $3}' )

# get table tablespace information
tablespace_info=$( /usr/lib/postgresql/${PG_VERSION}/bin/oid2name -U postgres -H localhost -p $PORT -s -q -x -t $1 )
tablespace_oid=$( echo $tablespace_info | awk '{print $5}' )

# get table information
table_info=$( /usr/lib/postgresql/${PG_VERSION}/bin/oid2name -U postgres -H localhost -d $DATABASE_NAME -p $PORT -q -t $1 -x -i -S )
table_file=$( echo $table_info | awk '{print $1}' )

if [ $# -eq 2 ]
then
    table_path=$( echo "PG_${PG_VERSION}_${CATALOG_VERSION_NO}/$database_oid/$table_file" )
else
    table_path=$( echo "base/$database_oid/$table_file" )
fi

# get table index information
index_info=$( /usr/lib/postgresql/${PG_VERSION}/bin/oid2name -U postgres -H localhost -d $DATABASE_NAME -p $PORT -q -x -i -S | grep ${1}_pkey )
index_file=$( echo $index_info | awk '{print $1}' )

if [ $# -eq 2 ]
then
    index_path=$( echo "PG_${PG_VERSION}_${CATALOG_VERSION_NO}/$database_oid/$index_file" )
else
    index_path=$( echo "base/$database_oid/$index_file" )
fi

# get table TOAST table
TOAST_oid=$( psql -t -A -U postgres -h localhost -d $DATABASE_NAME -p $PORT -c "select relfilenode from pg_class where oid = (select reltoastrelid from pg_class where relname = '"${1}\'");" )
TOAST_path=base/$database_oid/$TOAST_oid

#echo " ---------- database info ---------- "
#echo "oid: $database_oid"
#echo "tspc: $database_tablespace"

#echo " ---------- tablespace info ---------- "
#echo "oid: $tablespace_oid"

#echo " ---------- table info ---------- "
#echo "path: $PGDATA/$table_path"

echo "*---*---*---*---*---*---*---*---*---*---*---*---*---*---*---*---*"
echo "1) +++SELF+++ file of table		$1"
get_file_mac "$PGDATA/$table_path"
echo "2) +++PART+++ file of table		$1"
get_file_mac "$PGDATA/$table_path.1"
echo "3) +++FSM+++ file of table		$1"
get_file_mac "$PGDATA/${table_path}_fsm"
echo "4) +++VM+++ file of table			$1"
get_file_mac "$PGDATA/${table_path}_vm"
echo "5) +++INDEX+++ file of table		$1"
get_file_mac "$PGDATA/$index_path"
echo "6) +++TOAST+++ file of table		$1"
get_file_mac "$PGDATA/$TOAST_path"
#TODO: Проверка метки индекса TOAST таблицы. Проверка всех слоев TOAST
#echo "TOAST index"
#get_file_mac "$PGDATA/$TOAST_index"
echo "-----------------------------------------------------------------"
