create database policy_example;
\connect policy_example;

-- ### Создание таблиц ###
--
-- Создание таблицы с подразделениями
create table units 
(
    id integer primary key,
    name text,
    parent_id integer,
    role text
);
grant all on table units to public;
--
-- Создание таблицы с бойцами
create table warriors
(
    id serial primary key,
    fio text,
    unit_id integer,
    role text
);
grant all on table warriors to public;
--
-- ### Наполнение таблиц ###
--
-- Наполнение таблицы с подразделениями
insert into units (id, name, parent_id, role) 
values 
    (1, 'Подразделение 1', NULL,'p1'),

    (11, 'Подразделение 11', 1,'p11'),
    (111, 'Подразделение 111', 11,'p111'),
    (112, 'Подразделение 112', 11,'p112'),
    (113, 'Подразделение 113', 11,'p113'),

    (12, 'Подразделение 12', 1,'p12'),
    (121, 'Подразделение 121', 12,'p121'),
    (122, 'Подразделение 122', 12,'p122'),
    (123, 'Подразделение 123', 12,'p123'),

    (13, 'Подразделение 13', 1,'p13'),
    (131, 'Подразделение 131', 13,'p131'),
    (132, 'Подразделение 132', 13,'p132'),
    (133, 'Подразделение 133', 13,'p133');
--
-- Наполнение таблицы с бойцами
insert into warriors (fio, unit_id, role)
values
    ('ФИО 1', 1, 'b1'),
    ('ФИО 2', 1, 'b2'),
    ('ФИО 3', 1, 'b3'),
    ('ФИО 4', 11, 'b4'),
    ('ФИО 5', 11, 'b5'),
    ('ФИО 6', 11, 'b6'),
    ('ФИО 7', 12, 'b7'),
    ('ФИО 8', 12, 'b8'),
    ('ФИО 9', 12, 'b9'),
    ('ФИО 10', 13, 'b10'),
    ('ФИО 11', 13, 'b11'),
    ('ФИО 12', 13, 'b12'),
    ('ФИО 13', 111, 'b13'),
    ('ФИО 14', 111, 'b14'),
    ('ФИО 15', 111, 'b15'),
    ('ФИО 16', 112, 'b16'),
    ('ФИО 17', 112, 'b17'),
    ('ФИО 18', 112, 'b18'),
    ('ФИО 19', 113, 'b19'),
    ('ФИО 20', 113, 'b20'),
    ('ФИО 21', 113, 'b21'),
    ('ФИО 22', 121, 'b22'),
    ('ФИО 23', 121, 'b23'),
    ('ФИО 24', 121, 'b24'),
    ('ФИО 25', 122, 'b25'),
    ('ФИО 26', 122, 'b26'),
    ('ФИО 27', 122, 'b27'),
    ('ФИО 28', 123, 'b28'),
    ('ФИО 29', 123, 'b29'),
    ('ФИО 30', 123, 'b30'),
    ('ФИО 31', 131, 'b31'),
    ('ФИО 32', 131, 'b32'),
    ('ФИО 33', 131, 'b33'),
    ('ФИО 34', 132, 'b34'),
    ('ФИО 35', 132, 'b35'),
    ('ФИО 36', 132, 'b36'),
    ('ФИО 37', 133, 'b37'),
    ('ФИО 38', 133, 'b38'),
    ('ФИО 39', 133, 'b39');
--
-- Включение защиты на уровне строк
alter table units enable row level security;
alter table warriors enable row level security;
--
-- ### Создание ролей ###
--
-- Линейные роли
create role p1;
create role p11;
create role p111;
create role p112;
create role p113;
create role p12;
create role p121;
create role p122;
create role p123;
create role p13;
create role p131;
create role p132;
create role p133;
--
-- Иерархические роли
create role n1;
create role n11;
create role n111;
create role n112;
create role n113;
create role n12;
create role n121;
create role n122;
create role n123;
create role n13;
create role n131;
create role n132;
create role n133;
--
-- ### Распределение иерархических ролей ###
grant n1 to n11;
grant n1 to n12;
grant n1 to n13;
grant n11 to n111;
grant n11 to n112;
grant n11 to n113;
grant n12 to n121;
grant n12 to n122;
grant n12 to n123;
grant n13 to n131;
grant n13 to n132;
grant n13 to n133;
grant n1 to p1;
grant n11 to p11;
grant n111 to p111;
grant n112 to p112;
grant n113 to p113;
grant n12 to p12;
grant n121 to p121;
grant n122 to p122;
grant n123 to p123;
grant n13 to p13;
grant n131 to p131;
grant n132 to p132;
grant n133 to p133;
--
-- ### Создание ролей рядовых бойцов ###
\! useradd b1
\! createuser -h localhost -U postgres b1
\! pdpl-user -l 0:0 b1
\! useradd b2
\! createuser -h localhost -U postgres b2
\! pdpl-user -l 0:0 b2
\! useradd b3
\! createuser -h localhost -U postgres b3
\! pdpl-user -l 0:0 b3
\! useradd b4
\! createuser -h localhost -U postgres b4
\! pdpl-user -l 0:0 b4
\! useradd b5
\! createuser -h localhost -U postgres b5
\! pdpl-user -l 0:0 b5
\! useradd b6
\! createuser -h localhost -U postgres b6
\! pdpl-user -l 0:0 b6
\! useradd b7
\! createuser -h localhost -U postgres b7
\! pdpl-user -l 0:0 b7
\! useradd b8
\! createuser -h localhost -U postgres b8
\! pdpl-user -l 0:0 b8
\! useradd b9
\! createuser -h localhost -U postgres b9
\! pdpl-user -l 0:0 b9
\! useradd b10
\! createuser -h localhost -U postgres b10
\! pdpl-user -l 0:0 b10
\! useradd b11
\! createuser -h localhost -U postgres b11
\! pdpl-user -l 0:0 b11
\! useradd b12
\! createuser -h localhost -U postgres b12
\! pdpl-user -l 0:0 b12
\! useradd b13
\! createuser -h localhost -U postgres b13
\! pdpl-user -l 0:0 b13
\! useradd b14
\! createuser -h localhost -U postgres b14
\! pdpl-user -l 0:0 b14
\! useradd b15
\! createuser -h localhost -U postgres b15
\! pdpl-user -l 0:0 b15
\! useradd b16
\! createuser -h localhost -U postgres b16
\! pdpl-user -l 0:0 b16
\! useradd b17
\! createuser -h localhost -U postgres b17
\! pdpl-user -l 0:0 b17
\! useradd b18
\! createuser -h localhost -U postgres b18
\! pdpl-user -l 0:0 b18
\! useradd b19
\! createuser -h localhost -U postgres b19
\! pdpl-user -l 0:0 b19
\! useradd b20
\! createuser -h localhost -U postgres b20
\! pdpl-user -l 0:0 b20
\! useradd b21
\! createuser -h localhost -U postgres b21
\! pdpl-user -l 0:0 b21
\! useradd b22
\! createuser -h localhost -U postgres b22
\! pdpl-user -l 0:0 b22
\! useradd b23
\! createuser -h localhost -U postgres b23
\! pdpl-user -l 0:0 b23
\! useradd b24
\! createuser -h localhost -U postgres b24
\! pdpl-user -l 0:0 b24
\! useradd b25
\! createuser -h localhost -U postgres b25
\! pdpl-user -l 0:0 b25
\! useradd b26
\! createuser -h localhost -U postgres b26
\! pdpl-user -l 0:0 b26
\! useradd b27
\! createuser -h localhost -U postgres b27
\! pdpl-user -l 0:0 b27
\! useradd b28
\! createuser -h localhost -U postgres b28
\! pdpl-user -l 0:0 b28
\! useradd b29
\! createuser -h localhost -U postgres b29
\! pdpl-user -l 0:0 b29
\! useradd b30
\! createuser -h localhost -U postgres b30
\! pdpl-user -l 0:0 b30
\! useradd b31
\! createuser -h localhost -U postgres b31
\! pdpl-user -l 0:0 b31
\! useradd b32
\! createuser -h localhost -U postgres b32
\! pdpl-user -l 0:0 b32
\! useradd b33
\! createuser -h localhost -U postgres b33
\! pdpl-user -l 0:0 b33
\! useradd b34
\! createuser -h localhost -U postgres b34
\! pdpl-user -l 0:0 b34
\! useradd b35
\! createuser -h localhost -U postgres b35
\! pdpl-user -l 0:0 b35
\! useradd b36
\! createuser -h localhost -U postgres b36
\! pdpl-user -l 0:0 b36
\! useradd b37
\! createuser -h localhost -U postgres b37
\! pdpl-user -l 0:0 b37
\! useradd b38
\! createuser -h localhost -U postgres b38
\! pdpl-user -l 0:0 b38
\! useradd b39
\! createuser -h localhost -U postgres b39
\! pdpl-user -l 0:0 b39
--
grant p1 to b1;
grant p1 to b2;
grant p1 to b3;
--
grant p11 to b4;
grant p11 to b5;
grant p11 to b6;
--
grant p12 to b7;
grant p12 to b8;
grant p12 to b9;
--
grant p13 to b10;
grant p13 to b11;
grant p13 to b12;
--
grant p111 to b13;
grant p111 to b14;
grant p111 to b15;
--
grant p112 to b16;
grant p112 to b17;
grant p112 to b18;
--
grant p113 to b19;
grant p113 to b20;
grant p113 to b21;
--
grant p121 to b22;
grant p121 to b23;
grant p121 to b24;
--
grant p122 to b25;
grant p122 to b26;
grant p122 to b27;
--
grant p123 to b28;
grant p123 to b29;
grant p123 to b30;
--
grant p131 to b31;
grant p131 to b32;
grant p131 to b33;
--
grant p132 to b34;
grant p132 to b35;
grant p132 to b36;
--
grant p133 to b37;
grant p133 to b38;
grant p133 to b39;
--
-- Создание пользователей с расширенными правами
\! useradd "Психолог"
\! createuser -h localhost -U postgres "Психолог"
\! pdpl-user -l 0:0 "Психолог"
\! useradd "Кадровик"
\! createuser -h localhost -U postgres "Кадровик"
\! pdpl-user -l 0:0 "Кадровик"
\! useradd "Военнослужащий"
\! createuser -h localhost -U postgres "Военнослужащий"
\! pdpl-user -l 0:0 "Военнослужащий"
--
-- Распределение ролей конечным пользователям
grant "Психолог" to p11; -- Психолог
grant "Кадровик" to n11; -- Кадровик
grant "Военнослужащий" to b1; -- Военнослужащий
--
-- Функция получения ролей пользователя
create function user_roles(inp_username name) returns setof name as 
$$
BEGIN
    return query 
    with recursive r as (
        select 
            r1.rolname, 
            r1.oid, 
            m1.member
        from 
            pg_roles r1
            left outer join pg_auth_members m1 on m1.roleid = r1.oid
        where
            r1.rolname = inp_username

            union

        select
            r2.rolname,
            r2.oid,
            m2.member
        from
            r,
            pg_roles r2 
            left outer join pg_auth_members m2 on m2.roleid = r2.oid
        where 
            r2.oid = r.member
    )
    select rolname from r group by rolname;
END
$$
language 'plpgsql';
--
-- Функция проверки есть ли у пользователя данная роль
create function is_user_role(inp_username name, inp_rolename name) returns bool as 
$$
BEGIN
    return
    EXISTS (
        with recursive r as (
            select 
                r1.rolname, 
                r1.oid, 
                m1.member
            from 
                pg_roles r1
                left outer join pg_auth_members m1 on m1.roleid = r1.oid
            where
                r1.rolname = inp_username

                union

            select
                r2.rolname,
                r2.oid,
                m2.member
            from
                r,
                pg_roles r2 
                left outer join pg_auth_members m2 on m2.roleid = r2.oid
            where 
                r2.oid = r.member
        )
        select rolname from r where rolname = inp_rolename group by rolname
    );
END
$$
language 'plpgsql';
-- Создание политики контроля доступа к строкам
-- Предполагается, что доступ к строке таблицы имеет тот пользователь,
-- у которого есть роль, описанная в столбце roles этой строки.
create policy example on units for select 
using 
(is_user_role(current_user, role));

create policy example2 on warriors for select
using
(is_user_role(current_user, role));

