/* Тестирование освобождаемой памяти основывается на подмене вызова
 * функции free. В подменном варианте содержимое указателя и pid процесса
 * сохраняется в файл /tmp/out.txt, затем вызывается оригинальная ф-я free,
 * которая освобождает память.
 * Дескриптор файла out.txt нужно открыть в начале выполнения программы.
 * Для этого дополнительно подменяется системный вызов функции main. Подмена
 * использует изначальный системный вызов, но с main_hook вместо main. В функции 
 * main_hook открывается файл out.txt, затем вызывается оригинальная main
 * основной программы.
 * В файл вывода попадут данные всех дочерних процессов программы. Порядок записей
 * никак не гарантируется, для тестирования не требуется.
 * Вызов free и dlym может выполняться в других расширяемых библиотеках и приводить
 * к зацикливанию.
*/

#define _GNU_SOURCE
#include <stddef.h>
#include <dlfcn.h>
#include <stdio.h>
#include <malloc.h>
#include <unistd.h>

FILE *fd = 0;

/* Указатель на оригинальную ф-ю main() программы */
static int (*main_orig)(int, char **, char **);

/* Подменный main(), вызываемый в __libc_start_main() */
int main_hook(int argc, char **argv, char **envp)
{
	/* Открываем файл для сохранения данных */
	fd = fopen("/tmp/out.txt","w+");

	/* Вызываем оригинальную ф-ю main */
    int ret = main_orig(argc, argv, envp);

	/* Закрываем файл с данными */
	fclose(fd);

    return ret;
}

/* Подмена для __libc_start_main(), позволяющая заменить main */
int __libc_start_main(
    int (*main)(int, char **, char **),
    int argc,
    char **argv,
    int (*init)(int, char **, char **),
    void (*fini)(void),
    void (*rtld_fini)(void),
    void *stack_end)
{
    /* Сохраняем адрес функции main */
    main_orig = main;

    /* Находим изначальную __libc_start_main()... */
    typeof(&__libc_start_main) orig = dlsym(RTLD_NEXT, "__libc_start_main");

    /* и используем её с подменным вариантом main */
    return orig(main_hook, argc, argv, init, fini, rtld_fini, stack_end);
}

/* Подмена free */
void free(void *ptr)
{
	/* Указатель на оригинальную ф-ю free */
	static void (*libc_free)(void*) = 0;

	/* Получаем размер освобождаемой памяти */
	size_t ubsize = malloc_usable_size(ptr);

	/* Сохраняем данные (не очень оптимально) */
	if(fd)
	{
		int pid_d = getpid();
		fprintf(fd, "Process IDentifier = %d. Allocated %d bytes to free: ", pid_d, ubsize);
		for(int jj = 0; jj < ubsize; jj++){
			fprintf(fd,"%c", *((char *)(ptr+jj)));
		}
		fprintf(fd,"\n");
	}

	/* Находим оригинальную ф-ю free */   
	if(!libc_free)
		libc_free = dlsym(RTLD_NEXT, "free");

	/* Вызываем оригинальную ф-ю free */
	libc_free(ptr);
}
