--
-- Создание вспомогательных функций

-- Функция задержки с ожиданием результата
-- !!ВНИМАНИЕ!! 
-- Эта функция не является безопасной, она позволяет выполнить ЛЮБОЙ SQL код и 
-- допустима только в тестовой БД, подлежащей уничтожению после выполнения теста
create type advanced_waiting_type as (label text, result boolean);
create or replace function advanced_waiting(query text default null, 
									        expected_result integer default null, 
									        max_count integer default null, 
									        delay integer default null,
									        label text default null)
    returns advanced_waiting_type language plpgsql as
$advanced_waiting$
declare 
	result int := null;
begin
	--check parameters
	if query is null then
		return (label, false);
	end if;
	expected_result := coalesce(expected_result, 1);
	max_count := coalesce(max_count, 60);
    delay := coalesce(delay, 2);
    --execute query and check result
    for i in 1..max_count loop
	    execute query into result;
	    if result = expected_result then
	    	return (label, true);
	    end if;
	    perform pg_sleep(delay);
    end loop;
    return (label, false);
end
$advanced_waiting$;

--
-- Функция возвращает значения всех конфигурационных параметров в формате csv
-- для контроля настроек кластера перед выполнением теста
-- !!ВНИМАНИЕ!! 
-- Эта функция не является безопасной, она позволяет получить значения всех настроек кластера и 
-- допустима только в тестовой БД, подлежащей уничтожению после выполнения теста
create or replace function get_settings_csv(in delimiter char default null)
returns text language plpgsql as
$get_settings_csv$
declare
    rec record; 
    ret_value   text := NULL;
begin
	--check parameters
    delimiter := coalesce(delimiter, ';');
    --execute query and collecting parameter values
	for rec in
		select name,
			   setting
		from pg_catalog.pg_settings
		order by name
	loop
		if ret_value is null then
			ret_value := rec.name||delimiter||rec.setting;
		else
			ret_value := ret_value||e'\n'||rec.name||delimiter||rec.setting;
		end if;
	end loop;
    return ret_value;
end
$get_settings_csv$;